﻿using Newtonsoft.Json.Linq;
using System;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace QrZahlteilConsoleApiExample
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void cmdRun_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(txtComputerAppExe.Text)) {
                MessageBox.Show("Bitte geben Sie den Pfad zu QrZahlteilScanner.exe an.");
                return;
            }

            if (!File.Exists(txtComputerAppExe.Text)) {
                MessageBox.Show("Die Datei QrZahlteilScanner.exe konnte nicht gefunden werden.");
                return;
            }

            // QR-Zahlteil ComputerApp starten und Daten empfangen
            Task.Run(StartQrZahlteilComputerApp);
        }

        private void StartQrZahlteilComputerApp()
        {
            var psi = new ProcessStartInfo
            {
                FileName = txtComputerAppExe.Text,
                Arguments = txtComputerAppExeArgs.Text,
                UseShellExecute = false,
                RedirectStandardError = true,
                RedirectStandardOutput = true
            };

            var p = new Process();
            p.StartInfo = psi;
            p.OutputDataReceived += (_, args) =>
            {
                // Es wird null geliefert, wenn der Anwender die QR-Zahlteil ComputerApp schliesst, ohne dass Daten gescannt wurden
                if (args.Data == null) {
                    return;
                }
                InvokeIfRequired(this, () => { ShowOutput(args.Data); });
            };
            p.Start();
            p.BeginOutputReadLine();
            // Wenn der Anwender die QR-Zahlteil ComputerApp nicht innerhalb einer definierten Frist schliesst, wird angenommen, dass ein Problem aufgetreten ist.
            var timeout = TimeSpan.FromSeconds(120);
            if (!p.WaitForExit(Convert.ToInt32(timeout.TotalMilliseconds))) {
                throw new TimeoutException($"Program didn't exited after {timeout}");
            }

            // Sicherstellen, dass der Vorgang regulär beendet wurde
            if (p.ExitCode != 0) {
                InvokeIfRequired(this, () => { txtOutputBase64.Text = $"Program closed with ExitCode {p.ExitCode}"; });
            }

            // Behandlung, Falls Fehler in den StandardError Output geschrieben wurden
            string error = p.StandardError.ReadToEnd();
            if (!string.IsNullOrEmpty(error)) {
                throw new Exception($"Program returned error {error}");
            }
        }

        private void ShowOutput(string data)
        {
            // Empfangende Rohdaten anzeigen
            txtOutputBase64.Text = data;

            // Empfangende Daten im JSON Format dekodieren und anzeigen
            var decoded = FromBase64(data);
            txtOutputDecoded.Text = decoded;

            var json = JObject.Parse(decoded);
            // Rückgabewert prüfen, ob beim Scannen mölicherweise ein Fehler aufgetreten ist
            if (json["responseCode"].ToString() != "success") {
                txtOutputDecodedData.Text = $"responseCode: {json["responseCode"]}";
                return;
            }

            // Gescannte Daten aus der QR-Code im JSON Format dekodieren
            var decodedData = FromBase64(json["response"]["data"]["data_base64"].ToString());
            // Gescannte Daten aus dem QR-Code anzeigen. Zeilenumbrüche aus Unix Plattformen \n werden in \r\n geändert.
            txtOutputDecodedData.Text = decodedData.Replace("\n", Environment.NewLine);
        }

        private static void InvokeIfRequired(Control ctrl, MethodInvoker action)
        {
            if (ctrl.InvokeRequired) {
                ctrl.BeginInvoke(action);
            } else {
                action();
            }
        }

        private static string FromBase64(string value)
        {
            return Encoding.UTF8.GetString(Convert.FromBase64String(value));
        }

        private void lblWebsite_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Process.Start(lblWebsite.Text);
        }

        private void llDownloadComputerAppZip_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Process.Start($"{lblWebsite.Text}/#download");
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            lblTitel.Text += $@" Version {Assembly.GetExecutingAssembly().GetName().Version}";
        }
    }
}
